#!/bin/bash

# if groups are managed by Active Directory or LDAP
# 1 for yes, 0 for no
declare -r GROUPS_AD_MANAGED=1

# if Kerberos key distribution center is managed by active directory or not
# if yes, there is no need for creating principal and keytabs - this is done automatically by Active Directory, when new user is created
# 1 for yes, 0 for no
declare -r KDC_AD_MANAGED=1
declare -r HDFS_NAMESERVICE=nameservice1

function createUserFolder(){

	local username=$1

	groupName=$( echo "${username}" | awk '{print tolower($0)}' )
    groupName="${groupName//./_}"

    hdfs dfs -mkdir -p /user/${username} &&
	hdfs dfs -chown ${username}:${groupName} /user/${username} &&
    hdfs dfs -setfacl -m user:${username}:rwx /user/${username} || { ERRORT "could not create HDFS folder"; exit 1; }
    INFOT "folder has been created or already exists"
    setQuota "/user/${username}" 50

}

#executes an HDFS command
#serves as a wrapper for production deployment, where HDFS calls can differ (using sudo in front of commands and so on)
function execHDFS(){
    local command=$1

    sh ${HDFS_COMMAND_PREFIX} $command || {
        ERRORT "HDFS command $command could not be executed";
        return 1;
    }
    return 0;

}


function createWorkspaceHDFSFolder (){
    local workspaceName=$1
	local workspaceFolder=$2
    local workspaceType=$3

    workspaceGroup=$WORKSPACE_PREFIX$workspaceName


    execHDFS "hdfs dfs -mkdir -p ${workspaceFolder}" &&

    if [[ $workspaceType == 'group' ]]
    then
	   hdfs dfs -chown hdfs:${workspaceGroup} ${workspaceFolder} &&
       hdfs dfs -chmod 770 ${workspaceFolder} || {
            ERRORT "could not create HDFS folder"; exit 1;
       }
    else
       hdfs dfs -chown hdfs:hdfs ${workspaceFolder} || {
            ERRORT "could not create HDFS folder";
            exit 1;
       }
    fi

    INFOT "folder has been created or already exists"

}

function grantRoleToUserGroup() {
  local workspaceFolder=$1
  local userGroup=$2
  local workspaceName=$3

  url="${workspaceFolder}"
  roleName="${workspaceName//./_}";
  URIgrant="GRANT ALL ON URI 'hdfs://$HDFS_NAMESERVICE$url' TO ROLE $roleName;"

  makeQuery "GRANT ROLE $roleName TO GROUP `$userGroup`;" "GRANT role to group succeeded" "grant role to group failed" "impala" &&
  makeQuery "${URIgrant}" "GRANT ON URI SUCEEDED" "GRANT ON URI failed" "impala" &&
  makeQuery "GRANT ALL ON DATABASE wsp_$workspaceName TO ROLE $roleName WITH GRANT OPTION;" "GRANT on database to role succeeded" "GRANT on database to role failed" "impala" || {
    ERRORT "GRANT SETUP FAILURE OCURRED";
    exit 1;
  }
}

function createHdfsFolder (){
    # foldername - absolute path
    local folderName=$1

    # group, to which grant the access to this folder
    local groupName=$2

    hdfs dfs -mkdir -p ${folderName} &&
    hdfs dfs -chown hdfs:${groupName} ${folderName} &&
    hdfs dfs -setfacl -m group:${groupName}:rwx ${folderName} || { ERRORT "Could not create HDFS folder"; exit 1; }
    INFOT "folder has been created or already exists"

}

function setHDFSGrant (){

    # sets access to some folder for specified group

    local folderName=$1

    local groupName=$2

    local privilege=$3
    hdfs dfs -setfacl -m group:${groupName}:${privilege} ${folderName} || { ERRORT "Could not assign HDFS ACL for folder ${folderName}"; exit 1; }
    INFOT "ACL access has been set up for folder ${folderName} and group name ${groupName}"

}


function createDatabase(){

    # creates a database

    local databaseName=$1

    databaseName=$( echo "${databaseName}" | awk '{print tolower($0)}' )
    databaseName="${databaseName//./_}"

    makeQuery "CREATE DATABASE IF NOT EXISTS ${databaseName};" "DATABASE $databaseName created or already exists" "Database $databaseName could not been created" "impala"
    INFOT "database has been created"

}



# makes query to hive or impala
function makeQuery () {
	dotaz=$1
	success_message=$2
	fail_message=$3
	service=$4

    IMPALAHOST=node1

	hivePrincipal="hive/edge1.datalab.adastracorp.net@DATALAB.ADASTRA.CZ";
	hiveServer="edge1.datalab.adastracorp.net";
	HIVEHOST="jdbc:hive2://$hiveServer:10000;default;principal=$hivePrincipal"

	beeline_connection="${HIVEHOST}"
	impala_server="${IMPALAHOST}"

	echo $dotaz

	case "$service" in
			hive)

			beeline -u "$beeline_connection" -e "$dotaz" &> hive-log.txt || { ERRORT "$fail_message"; return 1; }
				INFOT "$success_message"
			;;
			impala)
				impala-shell -i $impala_server -q "$dotaz" &> impala-log.txt || { ERRORT "$fail_message"; return 1; }
				INFOT "$success_message";
			;;
			impalaNoFile)
				impala-shell -i $impala_server -q "$dotaz" 2> impala-log.txt || { ERRORT "$fail_message"; return 1; }
				INFOT "$success_message";
			;;
			*)
				impala-shell -i $impala_server -q "$dotaz" &> impala-log.txt || { ERRORT "$fail_message"; return 1; }
				INFOT "$success_message";
			;;

		esac
	return 0;
}

function createProjectRole() {

    roleName=$1
    dbName=$2
    privilege=$3
    uri=$4

    createDBrole $roleName
    grantOnDB $dbName $roleName ${privilege}
    grantOnURI $roleName $uri ${privilege}



}

function grantOnDB() {

    dbName=$1
    roleName=$2
    privilege=$3

    makeQuery "GRANT ${privilege} ON DATABASE ${dbName} TO ROLE ${roleName} WITH GRANT OPTION;" "GRANT on database to role succeeded" "GRANT on database to role failed" "impala" || {
        ERRORT "Grant on database failed, exiting";
        ERRORT "Bad query is: GRANT $privilege ON DATABASE ${dbName} TO ROLE ${roleName} WITH GRANT OPTION;";
        exit 1;
    }
    INFOT "Grant on database has succeeded";

}

# grants accces to some uri for some role name
# roleName = name of role
# uri = uri, on which the grant is applied
# privilege = which privilege will be applied
function grantOnURI(){

    roleName=$1
    uri=$2
    privilege=$3

    echo $uri
    echo $privilege
    echo $roleName


	URIgrant="GRANT ALL ON URI 'hdfs://$HDFS_NAMESERVICE$uri' TO ROLE $roleName;" &&
	makeQuery "${URIgrant}" "GRANT ON URI SUCEEDED" "GRANT ON URI failed" "impala" || {
        ERRORT "Bad query: ${URIgrant}";
		ERRORT "GRANT SETUP FAILURE OCURRED, EXITING SCRIPT";
		exit 1;
    }

}

# creates a database role
function createDBrole() {

    # group has to exist

    # TODO: check if group exist

    IMPALAHOST=node1

    roleName=$1

    # if role is not granted to any group
    noGrant=$2

    roleName="${roleName//./_}";

    query="SHOW ROLES;"
	output=$( impala-shell -i $IMPALAHOST -q "${query}" -B --quiet )


	roleExists=1

	while read -r role ; do


		if [[ "$role" == "$roleName" ]]; then
			roleExists=0 #yes, the role exists
			break;
		fi
	done <<< "$output"

    if [[ $roleExists -eq  1 ]]
	then
	    makeQuery "CREATE ROLE $roleName;" "ROLE $roleName created" "ROLE $roleName could not been created" "impala"

        if [[ $noGrant -ne 'y' ]]
        then
            makeQuery "GRANT ROLE $roleName TO GROUP $roleName;" "GRANT role to group succeeded" "grant role to group failed" "impala"

        fi


	else
    	    INFOT "ROLE $roleName already exists."
    fi

}

# creates database role and grants access to the database to that role
function createRole () {
    local roleName=$1
	local roleType=$2

    IMPALAHOST=worker2
    impala_server="${IMPALAHOST}"

    roleName=$( echo "${roleName}" | awk '{print tolower($0)}' )

    createDBrole ${roleName}_write
		createDBrole ${roleName}_read

	url="/user/${userName}"

	if [[ $roleType == "wsp" ]];
	then
		url="/appl/wsp_$roleName"
	fi

	makeQuery "GRANT ROLE ${roleName}_write TO GROUP ${roleName}_write;" "GRANT role to group succeeded" "grant role to group failed" "impala" &&
	makeQuery "GRANT ROLE ${roleName}_read TO GROUP ${roleName}_read;" "GRANT role to group succeeded" "grant role to group failed" "impala" &&
	URIgrant="GRANT ALL ON URI 'hdfs://$HDFS_NAMESERVICE$url' TO ROLE ${roleName}_write;" &&
	makeQuery "${URIgrant}" "GRANT ON URI SUCEEDED" "GRANT ON URI failed" "impala" &&
	makeQuery "GRANT ALL ON DATABASE ${roleName}_write TO ROLE ${roleName}_write WITH GRANT OPTION;" "GRANT on database to role succeeded" "GRANT on database to role failed" "impala" || {
		ERRORT "GRANT SETUP FAILURE OCURRED";
		exit 1;

	}

}

function grantLinuxFolderPermission() {
    #group and folder have to exist firstly
    echo "granting permission for linux folder"
  local group="$1"
  local folder="$2"

  #checkGroupExists $group "User group does not exist"


  setfacl -m g:$group:rw ${folder} || {
        ERRORT "Could not set permission to a linux folder $folder for group $group, exiting script";
        exit 1;
  }
}

# creates user home directory on the servers - on the OS
# adds user to primary group same as his name on all servers - because impala grants are then applied on this group
function makeUserHome(){

	username=$1

	groupName=$( echo "${username}" | awk '{print tolower($0)}' )
   	groupName="${groupName//./_}"

	checkGroupExists ${groupName} "User primary group does not exist"


	servers=(`echo ${SERVERS}`);

	for server in "${servers[@]}"; \

		do \
			INFOT "creating user home on ${server}"
			ssh ${server} bash -c "'
				echo ${server}
				mkdir -p /home/${username}

				if [[ $GROUPS_AD_MANAGED == 0 ]];
				then
					groupadd -f ${groupName} && # group cannot contain dot in its name
					usermod -a -G ${groupName} ${username} || {
						echo "Could not assign user to his primary group";
						exit 1;
					}
				fi


				chown ${username}:${groupName} /home/${username} || {
					ERRORT "Could not set up user home or group on server ${server}, exiting script";
					exit 1;
				}

			'"
		done
}

function checkGroupExists (){
	groupName=$1
	errorMsg=$2



	test=$( getent group ${groupName} )
	if [[ $test == "" ]];
    	then
        	{
				ERRORT "$errorMsg";
				exit 1;
			}
    fi

}

function kinitForCreateOps () {

    kinit hdfs/service@DATALAB.ADASTRA.CZ -kt hdfs.keytab || {
        ERRORT "Could not perform kinit";
        exit 1;

    }

}



# creates a folder for workspace
# if workspace is a group workspace, then
# workspace group has to be added firstly in SkoNet

#params:
#folder, name, type
function createWorkspaceLinuxFolder (){
	local workspaceFolder="$1"

    local workspaceName=$2
    # either group or poc
    local workspaceType=$3

	workspaceName=$( echo "${workspaceName}" | awk '{print tolower($0)}' )

    workspaceGroup=${WORKSPACE_PREFIX}${workspaceName}

	servers=(`echo ${SERVERS}`);


    if [[ $workspaceType == 'group' ]]
    then
        checkGroupExists ${workspaceGroup} "workspace group ${workspaceGroup} does not exist, exiting"
    fi

    echo ${workspaceFolder}

	for server in "${servers[@]}";
		do

			sudo ssh ${server} bash -c "'

            echo ${workspaceFolder}

			mkdir -p ${workspaceFolder}
            if [[ $workspaceType == "group" ]]
            then
                #chown the folder for the workspace access group
                chown root:${workspaceGroup} ${workspaceFolder}
            else
                #the access into the poc workspace folder is governed by ACLs, no default workspace group exists
                chown root:root ${workspaceFolder}
            fi


			chmod 770 ${workspaceFolder}
			'"	|| {
				ERRORT "error creating workspace folder, exiting script";
                exit 1;
			}
			INFOT "workspace folder has been created successfully on server ${server}"

        done

}


# not applicable - will be managed through SkoNet
function gainAccessWorkspace () {

	local username=$1

	local workspaceName=$2

	workspaceName=$( echo "${workspaceName}" | awk '{print tolower($0)}' )

	servers=( edge1 head1 head2 management dsc worker1 worker2 worker3 worker4 worker5 worker6 worker7 )

	INFOT "Assigning user $username to workspace $workspaceName"

	for server in "${servers[@]}"; \
		do \
			ssh ${server} bash -c "'
				usermod -a -G ${workspaceName} ${username}
			'"

		done
}



function generatePrivateKey (){
	username=$1

	groupName=$( echo "${username}" | awk '{print tolower($0)}' )
   	groupName="${groupName//./_}"


	if [ -e /home/${username}/.ssh/authorized_keys ]

	then
		INFOT "User SSH access is already set"
		return 0
	fi

	mkdir -p /home/${username}/.ssh
	chown ${username}:${groupName} /home/${username}/.ssh
	printf '\n\n\n' | su $username -c "ssh-keygen -t rsa"
	cp -ru /home/${username}/.ssh/id_rsa.pub /root/${username}.pub

	servers=( edge1 head1 head2 management node11 worker1 worker2 worker3 worker4 worker5 worker6 worker7 )

	for server in "${servers[@]}"
		do
			scp /root/${username}.pub root@${server}:/root

			ssh ${server} bash -c "'
				mkdir -p /home/${username}/.ssh &&
				chown ${username}:${groupName} /home/${username}/.ssh &&
				cp -ru /root/${username}.pub /home/${username}/.ssh/id_rsa.pub
				chown ${username}:${groupName} /home/${username}/.ssh/id_rsa.pub


				# if file authorized keys does not yet exist, then copy then content of the user public key to authorized_keys file
				if [ ! -f /home/${username}/.ssh/authorized_keys ]; then
					echo $( cat /home/${username}/.ssh/id_rsa.pub ) >> /home/${username}/.ssh/authorized_keys
					chown ${username}:${groupName} /home/${username}/.ssh/authorized_keys

				fi
				# set proper permissions
				chmod 600 /home/${username}/.ssh/authorized_keys
				chmod 700 /home/${username}/.ssh

			'"
		done

	#convert the file to ppk format from openSSH format and upload it into user HDFS directory
		#has to be kinited under HDFS admin first in order to be able to access user HDFS directory

	puttygen /home/${username}/.ssh/id_rsa -o /home/${username}/${username}.ppk
	hadoop fs -put -f "/home/${username}/${username}.ppk" /user/${username}/${username}.ppk
	hdfs dfs -chown ${username}:${groupName} /user/${username}/${username}.ppk


}



function generatePrincipal () {
	#Kerberos principal a keytab nebo heslo
	local username=$1 #jmeno uzivatele

	home=$2 #vychozi adresar pro home uzivatele (lisi se pro technicke uzivatele)

    KERBEROSREALM=DATALAB.ADASTRA.CZ

	HOST=root@head2

	ssh $HOST bash -c "'

		realm="@${KERBEROSREALM}"
		echo "addprinc -randkey $username$realm" | kadmin.local #vytvoreni principala &&
		echo "exit" | kadmin.local




	'"	|| { #exit interaktivni konzole
		ERRORT "Could not create principal or keytab for user $username";
		return 1;
	}

	#keytab je pote nutne presunout do slozky uzivatele
	# zmena prav na keytab a overeni spravnosti

}

function generateKeytab () {

	local username=$1

	groupName=$( echo "${username}" | awk '{print tolower($0)}' )
    groupName="${groupName//./_}"


	HOST=root@head2

	ssh $HOST bash -c "'

		KERBEROSREALM=DATALAB.ADASTRA.CZ

		realm="@${KERBEROSREALM}"


		#INFOT "Generating new keytab for user"

		echo "ktadd -norandkey -k $username.keytab $username$realm" | kadmin.local 2> /dev/null &&
		echo "exit" | kadmin.local
		kinit hdfs/service$realm -kt hdfs.keytab &&
		hdfs dfs -put -f $username.keytab /user/$username/$username.keytab
		hdfs dfs -chown $username:$groupName /user/$username/$username.keytab

		scp ${username}.keytab root@management:/root

	'"

	servers=( edge1 head1 head2 management dsc worker1 worker2 worker3 worker4 worker5 worker6 worker7 )

	for server in "${servers[@]}"; \
	do \

				scp /root/${username}.keytab root@${server}:/home/${username}
				ssh ${server} chown ${username}:${groupName} /home/${username}/${username}.keytab
	done
}





# creates user home folder on linux servers
function createUserHome(){

	servers=( $SERVERS )

	for server in "${servers[@]}"; \
		do \
			ssh ${server} bash -c "'
				mkdir -p /home/${username}

			'"
		done

}


function createSSHaccess(){

	groupName=$( echo "${1}" | awk '{print tolower($0)}' )
   	groupName="${groupName//./_}"

	servers=( "edge1" )



	for i in "${servers[@]}"
        do
			echo ${i}
		done
}
