#!/bin/bash

# ——————————————————————————
# Name: createUserStructures.sh
# Purpose: Create user structures on hadoop platform.
# Author: Jan Kodet
# ——————————————————————————
# Working directory:
DIR="`pwd`/"
# Global variables:
SCRIPT_NAME=`basename $0`
TIMESTAMP=`date +%F_%H:%M`

# Logging variables
LOGS_FOLDER="logs"
LOGS_FILE="logs.txt"
LOGGER_SCRIPT="logger.sh"
PROGNAME="user_management"
BASE_PATH="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# config
servers=( "edge1" )
home_folder_linux="/home/"
home_folder_hdfs="/usr/"
. $BASE_PATH/global_conf.config
. $BASE_PATH/$LOGGER_SCRIPT -l $LOGS_FOLDER$LOGS_FILE -f $PROGNAME -e "DEBUG"
OPTIND=1

. $BASE_PATH/appFramework.sh



# Naming prefixes
USR_DB_PREFIX="usr"

# ———————————————————————— help ——
# Display usage
#
help() {

  echo -e "\n# # # # # # # # # #"
  echo -e "  Script creates user structures on Hadoop platform.\n"
  echo "  First creates users home folder on linux on every node in cluster."
  echo -e "  After that home folder on HDFS is created\n"
  echo "  Lastly Hive database + role is assigned to user."
  echo -e " \n Usage -- createU -h for help, -n [name] for new user.\n"
  echo "# # # # # # # # # #";
exit 0
}

# ToDo - pridat home slozku do cesty
createUserHomeLinux() {

    for server in ${servers[@]}; do
      sudo ssh ${server} "mkdir -p ${home_folder_linux}$1 && chown $1:$1 ${home_folder_linux}$1"
      # sudo ssh ${server} "mkdir -p ${home_folder_linux}$1 && chown Jan.Kodet:jan_kodet ${home_folder_linux}$1"
      INFOT "User home created on ${server}."
    done

}

# ToDo - pridat home slozku do cesty
function createUserFolder(){

	local username=$1

	  groupName=$( echo "${username}" | awk '{print tolower($0)}' )
    groupName="${groupName//./_}"

    hdfs dfs -mkdir -p /usr/${username} &&
	  hdfs dfs -chown ${username}:${groupName} /usr/${username} &&
    hdfs dfs -setfacl -m user:${username}:rwx /usr/${username} || { ERRORT "could not create HDFS folder"; exit 1; }
    INFOT "folder has been created or already exists"
    setQuota "/usr/${username}" 50

}

function createHiveDB(){

    # creates a database

    local databaseName=$1

    databaseName=$( echo "${databaseName}" | awk '{print tolower($0)}' )
    databaseName="usr_${databaseName//./_}"

    makeQuery "CREATE DATABASE IF NOT EXISTS ${databaseName};" "DATABASE $databaseName created or already exists" "Database $databaseName could not been created" "impala"
    INFOT "database has been created"

}

createHiveRole () {

  INFOT "Creating Hive role for user $1 called role_$1"
  role=`echo $1 | awk '{print tolower($0)}'`
  role_final="role_${role//./_}"
  query="create role ${role_final};"
  INFOT "${query}"
  makeQuery "$query" "Role created successfully" "Role creation failed!" "impala"

}

createRightsToRole () {

    echo "Granting $1 right on database $2 to role $3"
    query="grant $1 on $2.* to role $3;"
    echo $query
    # makeQuery "$query" "Rights set to table successfully" "Rights to table wasn't set!" "impala"

}

assignRoleToUserGroup () {

    group_lower=$( echo "$1" | awk '{print tolower($0)}' )
    group="${$group_lower//./_}"
    role="role_$1"

    INFOT "Assigning $role to user group $group"
    query="grant role $role to group $group"
    INFOT $query
    makeQuery query "Role $role assigned to group $group successfully" "Role assignement failed!" "impala"

}

generatePrivateKey (){
	groupName=$( echo "${1}" | awk '{print tolower($0)}' )
  groupName="${groupName//./_}"


	if [ -e /home/${username}/.ssh/authorized_keys ]

	then
		INFOT "User SSH access is already set"
		return 0
	fi

	mkdir -p /home/${username}/.ssh
	chown ${username}:${groupName} /home/${username}/.ssh
	printf '\n\n\n' | su $username -c "ssh-keygen -t rsa"
	cp -ru /home/${username}/.ssh/id_rsa.pub /root/${username}.pub

	servers=( edge1 head1 head2 management node11 worker1 worker2 worker3 worker4 worker5 worker6 worker7 )

	for server in "${servers[@]}"
		do
			scp /root/${username}.pub root@${server}:/root

			sudo ssh ${server} bash -c "'
				mkdir -p /home/${username}/.ssh &&
				chown ${username}:${groupName} /home/${username}/.ssh &&
				cp -ru /root/${username}.pub /home/${username}/.ssh/id_rsa.pub
				chown ${username}:${groupName} /home/${username}/.ssh/id_rsa.pub


					# if file authorized keys does not yet exist, then copy then content of the user public key to authorized_keys file
				if [ ! -f /home/${username}/.ssh/authorized_keys ]; then
					echo $( cat /home/${username}/.ssh/id_rsa.pub ) >> /home/${username}/.ssh/authorized_keys
					chown ${username}:${groupName} /home/${username}/.ssh/authorized_keys

				fi
				# set proper permissions
				chmod 600 /home/${username}/.ssh/authorized_keys
				chmod 700 /home/${username}/.ssh

			'"
		done

	#convert the file to ppk format from openSSH format and upload it into user HDFS directory
		#has to be kinited under HDFS admin first in order to be able to access user HDFS directory

	puttygen /home/${username}/.ssh/id_rsa -o /home/${username}/${username}.ppk
	hadoop fs -put -f "/home/${username}/${username}.ppk" /user/${username}/${username}.ppk
	hdfs dfs -chown ${username}:${groupName} /user/${username}/${username}.ppk


}

kinitHdfs () {

    kinit hdfs/service@DATALAB.ADASTRA.CZ -kt /var/www/html/um/bashScripts/hdfs.keytab ||
    {
    ERRORT "Could not perform kinit"
    exit 1;
  }
}


# Parse params

while getopts ":h:u:" opt; do
    case ${opt} in
        h) help; exit 1;;
        u) username=${OPTARG};;
        *) help; exit 1;;
    esac
done
OPTIND=1

# ———————————————————————— main ——
# Main script processing
#
main() {
  INFOT "Starting script with following parameters - username = '${username}'.."
  INFOT "Performing kinit as HDFS"
  kinitHdfs &&
  createUserHomeLinux ${username}
  createUserFolder ${username}
  createHiveDB ${username}
  createDBrole ${username}
  createRightsToRole "all" "usr_${username}" "role_${username}" &&
  assignRoleToUserGroup ${username} &&
  INFOT "User structures has been created successfully!"
}
if [[ ! -z $username ]]; then
  main
else
  ERRORT "Username parameter required!"; exit 1
fi
